# $Id: ColorSpace.pm,v 1.6 1996/05/31 15:57:21 raman Exp $
package Pdf::ColorSpace;
require 5.000;
#use Pdf::ColorMap;
use Pdf::Obj;
use Pdf::Std qw( message_err f4 assertFloatRange );
use strict qw ( subs vars refs );
@Pdf::ColorSpace::ISA = qw( Pdf::Obj );
# {{{ Methods

BEGIN {
    $Pdf::ColorSpace::dimen{DeviceRGB} = 3;
    $Pdf::ColorSpace::dimen{DeviceCMYK} = 4;
    $Pdf::ColorSpace::dimen{DeviceGray} = 1;
    $Pdf::ColorSpace::dimen{CalRGB} = 3;
    $Pdf::ColorSpace::dimen{CalLab} = 3;
    $Pdf::ColorSpace::dimen{CalGray} = 1;
    $Pdf::ColorSpace::dimen{Index} = 1; # Not yet supported

    %Pdf::ColorSpace::default_cal_attribute = ( 'WhitePoint' => [ 0.95, 1, 0.9 ] );
}

sub new {
    my $class = shift;
    my $name = shift;
    defined($name) || message_err( 'Requires a name' );
    my $self = {};
    bless $self, $class;
    $self->setObjContents(
			Pdf::CosArray->new( Pdf::CosName->new( $name ),
					 Pdf::CosDict->new( 'WhitePoint', 'BlackPoint',
							   'Gamma', 'Matrix', 'Range' )
					   ) );
    $self->setArrayDict( @_ ) if defined( @_ );
    $self;
}

sub refType {
    'PdfColorSpace';
}

# Use to create a new object of type PdfColorSpace. Pass a reference
# to a Cos object. This method is written as a model of what is
# necessary to be done to objects that are read from a Pdf file, in
# order to categorise Pdf Objects.

# Method not yet used.

sub rebless {
    my $class = shift;
    my $contentref = shift;
    eval{ $contentref->refType } =~ /^CosArray/ ||
	message_err( 'PdfColorSpace requires a CosArray' );
    my $self = {};
    bless $self,$class;
    $self->setObjContents( $contentref );

    # Now figure out flag values
}

sub validate {
}

# Set values in Array Dict (ColorSpace Contents contains an array,
# with the second entry being a Dict).

sub setArrayDict {
    my $self = shift;
    my %p = @_;

    my $dict = @{$self->getObjContents()}[1];
    
    my($key,$val,@vlist);
    foreach $key (keys %p) {

	# All colorspace attributes take an array
	
	$val = $p{$key};
	ref($val) eq 'ARRAY' ||
	    message_err( "Colorspace attribute values must be passed as an array" );
	@vlist = @$val;
	
	# Verify arrays have good data, and set Dictionary entries

	if( $key =~ /^white/i ) {
	    assertFloat(@vlist) == $self->{Dimen} ||
		message_err( "Colorspace /WhitePoint array must contain $self->{Dimen} values" );
	    $dict->set( 'WhitePoint', [ @vlist ] );
	    
	} elsif( $key =~ /^black/i ) {
	    assertFloat(@vlist) == $self->{Dimen} ||
		message_err( "Colorspace /BlackPoint array must contain $self->{Dimen} values" );
	    $dict->set( 'BlackPoint', [ @vlist ] );
	    
	} elsif( $key =~ /^gamma/i ) {
	    assertFloat(@vlist) == $self->{Dimen} ||
		message_err( "Colorspace /Gamma array must contain $self->{Dimen} values" );
	    $dict->set( 'Gamma', [ @vlist ] );
		
	} elsif( $key =~ /^Matrix/i ) {
	    if( $self->{ColorSpace} eq 'CalRGB' ) {
		assertFloat(@vlist) == 9 ||
		    message_err( "Colorspace /Matrix array must contain 9 values" );
		$dict->set( 'Matrix', [ @vlist ] );
	    } else {
		message_err( "Colorspace /Matrix array only used for CalRGB" );
	    }		
		
	} elsif( $key =~ /^range/i ) {
	    if( $self->{ColorSpace} eq 'CalLab' ) {
		assertFloat(@vlist) == 4 ||
		    message_err( "Colorspace /Range array must contain 4 values" );
		$dict->set( 'Range', [ @vlist ] );
	    } else {
		message_err( "Colorspace /Range array only used for CalLab" );
	    }		

	} else {
	    message_err( "Undefined ColorSpace attribute '$key'" );
	}

    }	    
}	

# }}}
# {{{ package Pdf::File

package Pdf::File;

sub existColorSpace {
    1;
}

# Call when opening a new file. Sets up dummy objects for default
# color spaces. This provides a generic interface to find information
# about any colorspace (custom or default).

sub initColor {
    my $self = shift;

    # Initialise color space by defining objects for all default color
    # spaces. These objects are not added to $self->{Obj} (so that
    # they are not printed), and their Default bit is set (which means
    # that they are not custom color spaces). We need to do this so
    # that information about these color spaces is available thru a
    # generic interface.

    my($cs,$objref);
    foreach $cs ('DeviceRGB', 'DeviceCMYK', 'DeviceGray') {
	$objref = Pdf::ColorSpace->new( $cs );
        #print STDERR  " $Pdf::ColorSpace::dimen{$cs} = $cs\n"; ;
	$objref->setFlag( 'Default' => 1 );
	$objref->setFlag( 'Dimen' => $Pdf::ColorSpace::dimen{$cs} );
	$objref->setFlag( 'ColorSpace' => $cs );
	$self->{ColorSpace}{$cs} = $objref;
    }

    $self->{ColorSpaceIndex} = 0;

    # Define a palatte of initial colors
    
    $self->initColorPackage( $self->{pdf_color_package} );
}

# Define a new colorspace and create an object for this
# colorspace. Adds this object to file object list. Returns a unique
# name of the new colorspace (eg. CS1). This name should be used later
# when changing colorspace.  If passed with a minumum of parameters
# (eg. $file->defineColorSpace('CalRGB')) then default values for all
# parameters are used. The PDF WhitePoint parameter is not optional,
# so a default value for the WhitePoint is used.

sub newColorSpace {
    my $self = shift;
    my( $cspace, %p ) = @_;

    my $name = 'CS' . ++$self->{ColorSpaceIndex};
    my $dimen = $Pdf::ColorSpace::dimen{$cspace};

    # Verify that colorspace of custom color is a calibrated colorspace
    
    !defined( $self->{ColorSpace}{$cspace} ) ||
	message_err( "Colorspace '$cspace' is a default colorspace" );

    defined( $dimen ) ||
	message_err( "Colorspace '$cspace' is not a recognised calibrated colorspace" );

    my $cref = Pdf::ColorSpace->new( $name, @_ );
    $cref->setFlag( 'Dimen' => $dimen );
    $cref->setFlag ( 'ColorSpace' => $cspace );

    # Add the colorspace object to the array of file objects
    
    $self->addObjRef( $cref );

    # Return name and reference for new colorspace

    ( $name, $cref );
}

# }}}
# {{{ Emacs local variables  

### Local variables:
### major-mode: perl-mode
### perl-indent-level: 4
### voice-lock-mode: t
### folded-file: t
### End:

# }}}
1;
