# $Id: Graphic.pm,v 1.15 1996/05/31 16:02:59 raman Exp $
package Pdf::Graphic;
require 5.000;
use Pdf::Page;
use Pdf::Obj;
use Pdf::Gstate;
use Pdf::ColorSpace;
use Pdf::ColorMap;
use Pdf::Render;
use Pdf::Std qw( assertFloat assertFloatRange message_err
		f0 f1 f2 f3 f4 af0 af1 af2 af3 af4);
use strict qw ( subs vars refs );
@Pdf::Graphic::ISA = qw( Pdf::Obj );
# {{{ new refType pdfPrint

sub new {
    my $class = shift;
    my( $file, $pageref, $gcurrent, $gnext ) = @_;
    my $self = Pdf::Obj->new( 'Length', 'Filter', 'DecodeParms' );
    # bless $self, $class;
    $self = bless {
	"FileHandle" => $self->{FileHandle}
    }, $class;
    $self->setFileRef( $file );
    $self->setPageRef( $pageref );
    $self->setGstateRef( 'GCurrent', $gcurrent );
    $self->setGstateRef( 'GNext', $gnext );
    $self->clearPath;
    $self;
}

sub refType {
    'PdfGraphic';
}

# Set a reference to the Graphic stream's parent Page object.

sub setPageRef {
    my $self = shift;
    my $pageref = shift;
    eval{ $pageref->refType } eq 'PdfPage' ||
	message_err( 'Requires reference to existing PdfPage object' );
    $self->{PageRef} = $pageref;
}

sub getPageRef {
    my $self = shift;
    $self->{PageRef};
}

# Set a reference to the Graphic stream;s parent File object.

sub setFileRef {
    my $self = shift;
    my $objref = shift;
    eval{ $objref->refType } eq 'File' ||
	message_err( 'Requires reference to existing File object' );
    $self->{File} = $objref;
}

sub getFileRef {
    my $self = shift;
    $self->{FileRef};
}

# Set a reference to a Gstate object. Validates reference. To get
# reference use Pdf::Obj::getFlat().

sub setGstateRef {
    my $self = shift;
    my $key = shift;
    my $objref = shift;
    eval{ $objref->refType } eq 'Gstate' ||
	message_err( 'Requires reference to existing Gstate object' );
    $self->{$key} = $objref;
}

# Set Dictionary entries

sub setDict {
    my $self = shift;
    my( %p ) = @_;

    my $key;
    foreach $key (keys %p) {
	
	if( $key =~ /^fil/i ) {
	    if( !ref($p{$key}) ) {
		$self->setObjDict( 'Filter', Pdf::CosName( $p{$key} ) );

	    } elsif( ref($p{$key}) eq 'ARRAY' || message_err( "'$key' must be an array" ) ) {
		my $i;
		foreach $i (0.. $#{$p{$key}}) {
		    $self->pushObjDictArray( 'Filter', Pdf::CosName( @{$p{$key}}[$i] ) );
		}
	    }
	    
	} elsif( $key =~ /^dec/i ) {
	    message_err( "Code to support DecodeParms not yet written" );
	    
	} else {
	    message_err( "Unrecognised parameter '$key'" );
	}
    }
}

# }}}
# {{{ Stream printing

# Which method is selected determines whether it matters if text
# blocks are open or closed.

# Prints to the stream. Checks that stream has not been closed.

sub anyPrint {
    my $self = shift;
    !($self->{Done}) || message_err( 'Stream is closed' );
    if( $self->{Buffer} ) {
	$self->{Stream}->printStream( join( '', @_ ) );
    } else {
	$self->{File}->pdfPrint( @_ );
    }
}

# Prints to the stream. Checks that stream has not been closed. Checks
# that Text Block is not open.

sub strPrint {
    my $self = shift;
    !($self->{Done}) || message_err( 'Stream is closed' );
    !($self->{TextBlock}) || message_err( 'Text Block open' );
    if( $self->{Buffer} ) {
	$self->{Stream} .= join( '', @_ );
    } else {
	$self->{File}->pdfPrint( @_ );
    }
}

# Prints to the stream. Checks that stream has not been closed. Checks
# that Text Block is open.

sub textPrint {
    my $self = shift;
    !($self->{Done}) || message_err( 'Stream is closed' );
    ($self->{TextBlock}) || message_err( 'Text Block not open' );
    if( $self->{Buffer} ) {
	$self->{Stream} .= join( '', @_ );
    } else {
	$self->{File}->pdfPrint( @_ );
    }
}

# }}}
# {{{ endGraphic routines

sub endGraphic {
    my $self = shift;
    
    $self->{path} == 0 || message_err( 'Drawing path open' );
    $self->{Done} != 1 || message_err( 'Graphic not open' );
    $self->{Done} = 1;
    $self->{TextBlock} != 1 || message_err( 'Text block open' );

    # Flush any pending gstate changes before closing the stream.
    # This is only done if Pdf::File object global variable
    # pdf_stream_flush_gstate is 1.

    if( $self->{File}->{pdf_stream_flush_gstate} ) {
	$self->{GNext}->update( $self->{GCurrent}, $self );
    }

    # Close off the object.

    my $stream_length;
    if( $self->{Print} ) {
	if( $self->{Buffer} ) {
	    $stream_length = $self->doFilter;
	} else {
	    $stream_length = $self->{File}->{ByteCount} - $self->{StartByte};
	}
	
	$self->{LengthObj}->setObjContents( Pdf::CosInteger->new( $stream_length ) );
	$self->printObj( $self->{File} );
	$self->{File}->pdfPrint( "endstream\n" );
	$self->{File}->endObj;
	$self->{LengthObj}->printObj( $self->{File} );

    } else {

	( $self->{Buffer} == 1 ) || Message_err( 'Internal - Buffer flag does not agree with code' );
	
	# If there are no filters then doFilter will just return the
	# length of the buffer

	$stream_length = $self->doFilter;
	$self->setObjDict( 'Length', Pdf::CosInteger( $stream_length ) );
    }
    $self->updatePageResource( $self->{PageRef} );
}

# Same as endGraphic() except will close all open text streams and
# streams without complaining. This is most likely used only under
# fatal error conditions where the pdf file needs to be correctly
# written out, but an error has occured during processing of data.

sub forceEndGraphic {
    my $self = shift;

    $self->endText() if( $self->{TextBlock} );
    $self->endGraphic() if( !$self->{Done} );
    $self->updatePageResource( $self->{PageRef} );
}


# Add a stream to the specified page numbers. You can do this at any
# time, once the pages and stream have been created.

sub addGraphic {
    my $self = shift;
    my @pagelist = @_;

    my($pagenum,$pageref);
    foreach $pagenum (@pagelist) {
	$pageref = $self->{File}->getPageRef( $pagenum );
	$pageref->addContent( $self );
	$self->updatePageResource( $pageref );
    }
}

# Executes stream filters on the stream buffer. Returns the length of
# the stream buffer.

sub doFilter {
    my $self = shift;

    my $fref = $self->getObjDict( 'Filter' );
    if( defined( $fref ) ) {
	message_err( 'Filtered streams not yet supported' );
    } else {
	length( ${$self->{Stream}} );
    }
}
	
# }}}
# {{{ beginText endText

# The begin and end text routines should be placed around blocks of
# text rendering. As a shortcut you can pass an initial font type and
# font size into beginText. Subsequently, within the same text block,
# the font can be changed with a call to setFont. In both cases the
# font name is of the form 'Helvetica' and the size is in
# points. $fontsize is an optional parameter and, if not specified,
# will default to 1 (see description of &setFont routine).

sub beginText {
    my $self = shift;
    my(@p) = @_;

    $self->getPath && message_err( 'Cannot begin text block within a path' );
    ( $self->{Done} != 1 ) || message_err( 'Graphic is not open' );
    $self->{TextBlock} != 1 || message_err( 'Text Block already open' );
    $self->{TextBlock} = 1;
    
    $self->{TextX} = $self->{TextY} = 0; # Coordinates get reset for each BT
    $self->{TextXDiff} = 0;	# Amount of x leading (see T*)
    $self->textPrint( "BT\n" );

    # set ProcSet Text as used in Page

    $self->useProcSet( 'Text' );

    eval{ $self->{File}->existFont } ||
	message_err( 'Module Font.pm not loaded' );

    $self->setFont(@p) if @p;
}

sub endText {
    my $self = shift;

    $self->{TextBlock} == 1 || message_err( 'Text Block not open' );
    $self->{TextBlock} = 0;

    # $pdf_font_defined = 0; -- verify if this is still needed
    $self->strPrint( "ET\n" );
}

# }}}
# {{{ Resource Flags

# Flags are set to indicate which resources are use. Resource
# dictionary is associated with a Page, not a Graphic object: however
# if this stream is used elsewhere (using addGraphic()), then the
# Resource list for the stream must be checked and added to the
# resource list for the Page to which the graphic stream is added. All
# these resource elements are added to the Page by addGraphic() or
# endGraphic().

# $name must be one of 'PDF', 'Text', 'ImageB', 'ImageC', 'ImageI'

sub useProcSet {
    my $self = shift;
    my $name = shift;
    $self->{ProcSet}{$name} = 1;
}

# Adds {Font}{F1} = $fontref to PdfPage object. $name must be in form
# 'Helvetica'.

sub useFont {
    my $self = shift;
    my $name = shift;
    defined( my $fontref = $self->{File}->getFontRef( $name ) ) ||
	message_err( "Font '$name' not registered for use" );
    $self->{Font}{$fontref->getName()} = $fontref;
}

# Adds XObject.

sub useXObject {
    my $self = shift;
    my $name = shift;
    message_err( 'XObjects not yet supported' );
}

# Copies the Graphic Stream resources to the Page resources. Does not
# validate data (this is done when Page does a buildResource).

sub updatePageResource {
    my $self = shift;
    my $pageref = shift;

    my($type,$key);
    foreach $type ( 'ProcSet', 'Font', 'XObject' ) {
	foreach $key (keys %{$self->{$type}} ) {
	    $pageref->{$type}{$key} = $self->{$type}{$key};
	}
    }
}

# }}}
# {{{ package Pdf::Length

# A special object for dealing with defered stream lengths.

package Pdf::Length;
@Pdf::Length::ISA = qw( Pdf::Obj );

sub new {
    my $class = shift;
    my $objnum = shift;
    my $self = Pdf::Obj->new();
    bless $self, $class;
    $self->setObjNum( $objnum );
    $self;
}

# }}}
# {{{ To eventually be moved to File.pm

package Pdf::File;

sub existGraphic {
    1;
}

# Multiple streams can be open at the same time. A stream can be
# included in the contents of multiple pages, however beginGraphic
# takes only one page as a parameter (or defaults to
# $file->{PageIndex} if no page is specified). To add this stream to
# other pages use method $graphic->addGraphic($pagenum). If adding a
# stream to multiple pages then it is up to the user to ensure that
# gstate is consistent so that the streams display identically on all
# pages. See the stream documentation for details on how to achieve
# this consistency (I used to do these tricks automatically, but found
# this more difficult to maintain then it is worth).

# Usage: $streamref = $file->beginGraphic( $pagenum, 'Filter' => 'LzwDecode' )

sub beginGraphic {
    my $self = shift;

    # If the first parameter is a number then use that as the page number,
    # otherwise use $self->{PageIndex}.

    my $pagenum;
    if( $_[0] =~ /^\d+$/ ) {
	$pagenum = shift;
    } else {
	$pagenum = $self->{PageIndex};
    }
    my $pageref = $self->getPageRef( $pagenum );

    my %p = @_;

    my $graphic = Pdf::Graphic->new( $self, $pageref,
				    $pageref->{GCurrent}, $pageref->{GNext} );

    $pageref->addContent( $graphic );
    $graphic->setDict( %p );
    
    # set ProcSet PDF as used in Page

    $graphic->useProcSet( 'PDF' );

    # If the pdf file is already open and there are no filters to be
    # executed then immediately print stream object header, including
    # reference to following object which will contain the length of
    # the stream.

    if( $self->{Print} ) {
	$graphic->setObjNum( $self->beginObj );
	$graphic->{Print} = 1;
	$graphic->{LengthObj} = Pdf::Length->new( $self->reserveObj );
	$graphic->setObjDict( 'Length', Pdf::CosRef->new( $graphic->{LengthObj} ) );
	$graphic->getObjDictRef( 'Length' )->printObj( $self );
	$self->pdfPrint( "\nstream\n" );
	$graphic->{StartByte} = $self->{ByteCount};

    # Do not print stream at this time.  We need to reserve an object
    # number because the object number will be wanted by Pdf::Page
    # objects when they are printed, and Page objects are printed before
    # any late objects.

    } else {
	$graphic->setObjNum( $self->reserveObj );
	$self->addObjRef( $graphic );
    }

    # Stream contents will be buffered if this is a delayed print, or
    # if a filter will be applied to the contents. $graphic->{Buffer}
    # indicates whether stream is buffered.

    if( $self->{Print} && !defined($graphic->getObjDict( 'Filter' ) ) ) {
	$graphic->setFlag( 'Buffer', 0 );
    } else {
	$graphic->setFlag( 'Buffer', 1 );
	$graphic->{Stream} = Pdf::CosStream->new;
    }

    $graphic;		# return reference
}

# }}}
# {{{ Emacs local variables  

### Local variables:
### major-mode: perl-mode
### perl-indent-level: 4
### voice-lock-mode: t
### folded-file: t
### End:

# }}}
1;
