# $Id: Gstate.pm,v 1.14 1996/05/31 16:08:58 raman Exp $
package Pdf::Gstate;
use Pdf::ColorSpace;
use Pdf::Std qw( message_err f3 f4 );
use strict qw ( subs vars refs );
# {{{ Graphics State

BEGIN { 
    @Pdf::Gstate::gstate_list = ( 'w', 'i', 'j', 'J', 'M' );
    @Pdf::Gstate::tstate_list = ( 'Tc', 'Tw', 'Tz', 'TL', 'Ts', 'Tr' );
}

sub new {
    my $class = shift;
    my $self = {};
    bless $self, $class;
}

sub refType {
    'Gstate';
}

# Sets all gstate values to defaults. If a gstate reference is passed
# then sets all gstate values in the reference to defaults. Usage:
# $newgstate = gstate->init() generates a new object. $oldgref =
# gstate->init( $oldgref ) sets values of $oldgref to defaults.

sub init {
    my $self = shift;
    
    $self->{gstate}{w} = 1;	# Linewidth
    $self->{gstate}{i} = 0;	# Flatness
    $self->{gstate}{j} = 0;	# Line join
    $self->{gstate}{J} = 0;	# Line cap
    $self->{gstate}{d} = '[] 0'; # Dash pattern
    $self->{gstate}{M} = 10;	# Miter limit

    my $k;
    foreach $k ('stroke','fill') {
	$self->{$k}{name} = 'black';
	$self->{$k}{cspace} = 'DeviceGray';
	$self->{$k}{value} = [ 0 ];
    }

    $self->{tstate}{Tc} = 0;	# Character spacing
    $self->{tstate}{Tw} = 0;	# Word spacing
    $self->{tstate}{Tz} = 100;	# Horizontal scaling
    $self->{tstate}{TL} = 0;	# Text leading
    $self->{tstate}{Ts} = 0;	# Text rise
    $self->{tstate}{Tr} = 0;	# Text rendering mode

    $self->{fontname} = '';	# Font name (no default)
    $self->{fontsize} = -1;	# Font size (no default)

    $self->{GChange} = $self->{CChange} = $self->{TChange} = 0;

    $self;
}

# Copies gstate objects. If a second reference is passed then uses
# that reference as the pointer to the new object, rather then making
# a new object.

sub copy {
    my $self = shift;
    my($newref) = shift;

    my $new = 0;
    if( !defined($newref) ) {
	$newref = new Pdf::Gstate;
	$new = 1;
    }

    $newref->{gstate}{w} = $self->{gstate}{w};
    $newref->{gstate}{i} = $self->{gstate}{i};
    $newref->{gstate}{j} = $self->{gstate}{j};
    $newref->{gstate}{J} = $self->{gstate}{J};
    $newref->{gstate}{d} = $self->{gstate}{d};
    $newref->{gstate}{M} = $self->{gstate}{M};

    my $k;
    foreach $k ('stroke','fill') {
	$newref->{$k}{name} = $self->{$k}{name};
	$newref->{$k}{cspace} = $self->{$k}{cspace};
	$newref->{$k}{value} = [ @{$self->{$k}{value}} ];
    }

    $newref->{tstate}{Tc} = $self->{tstate}{Tc};
    $newref->{tstate}{Tw} = $self->{tstate}{Tw};
    $newref->{tstate}{Tz} = $self->{tstate}{Tz};
    $newref->{tstate}{TL} = $self->{tstate}{TL};
    $newref->{tstate}{Ts} = $self->{tstate}{Ts};
    $newref->{tstate}{Tr} = $self->{tstate}{Tr};

    $newref->{fontname} = $self->{fontname};
    $newref->{fontsize} = $self->{fontsize};

    $newref->{GChange} = $self->{GChange};
    $newref->{CChange} = $self->{CChange};
    $newref->{TChange} = $self->{TChange};
    
    $newref;
}

# Will immediately write any differences between next gstate and
# current gstate to pdf file. Uses a global (to package) variable
# $gstate_change and $cstate_change to check if there are any
# potential changes. These variables will shortcut the otherwise
# costly task of checking every gstate entry individually to check for
# differences. If a third parameter 'notstate' is provided then tstate
# will not be updated (this is useful for mark operators that do not
# require tstate to be up to date).  Does not assert for valid data in
# $next object. All assertions should occur when variables are set. We
# want quick returns from this function if it is not needed, because
# it is called for every show/mark operation.

sub update {
    my $next = shift;
    my $current = shift;
    my $graphic = shift;

    return if( !( $next->{GChange} || $next->{CChange} || $next->{TChange} ) );

    my(@p) = @_;		# Could be 'notstate' or 'nogstate'
    my $k;

    ( $current && $next ) || message_err( 'Requires two reference parameters' );
    defined($graphic) || message_err( 'Requires stream specifier' );
    
    if( $next->{GChange} && $p[0] ne 'nogstate' && $p[1] ne 'nogstate' ) {

	foreach $k (@Pdf::Gstate::gstate_list) {
	    if( $current->{gstate}{$k} != $next->{gstate}{$k} ) {
		$graphic->printGstate( $k );
		$current->{gstate}{$k} = $next->{gstate}{$k};
	    }
	}
	if( $current->{gstate}{d} ne $next->{gstate}{d} ) {
	    $graphic->printGstateD( );
	    $current->{gstate}{d} = $next->{gstate}{d};
	}
	$next->{GChange} = 0;
    }

    if( $next->{CChange} && $p[0] ne 'nocstate' && $p[1] ne 'nocstate' ) {

	my $sf;
	foreach $sf ('stroke','fill') {

	    # jpravetz - 9apr96 - fixed if{}else{} below

	    # When comparing GNext vs GCurrent colors, all of
	    # Colorname, Colorspace and Colorvalue must be defined
	    # (these would be set by Render.pm's setColor()). We only
	    # need to determine if colors are different
	    # here. Pdf::Render::printColor() will take care of
	    # figuring out what needs to be output.

#  	    print( "Comparing colors GCurrent=[", $current->getColorSpace($sf), " (",
#  		  join(', ',@{$current->getColorValue($sf)}), ")], GNext=[", $next->getColorSpace($sf), " (",
#  		  join(', ',@{$next->getColorValue($sf)}), ")] compare=",
# 		  arrayEqual( $current->getColorValue($sf), $next->getColorValue($sf) ), "\n" );

	    if( $next->getColorName($sf) eq '' ) {

		# Harder: Colors are defined by ColorSpace and
		# ColorValue, so compare these.

		if( $current->getColorSpace($sf) ne $next->getColorSpace($sf)
		   || !(arrayEqual( $current->getColorValue($sf), $next->getColorValue($sf) )) ) {
		    $graphic->printColor( $sf );
		}

	    } else {

		# Easier: Colors are defined by ColorName, so compare
		# these.

		if( $current->getColorName($sf) ne $next->getColorName($sf) ) {
		    $graphic->printColor( $sf );
		}
	    }
	}
	$next->{CChange} = 0;
    }

    if( $next->{TChange} && $p[0] ne 'notstate' && $p[1] ne 'notstate' ) {

	foreach $k (@Pdf::Gstate::tstate_list) {
	    if( $current->{tstate}{$k} != $next->{tstate}{$k} ) {
		$graphic->printTstate( $k );
		$current->{tstate}{$k} = $next->{tstate}{$k};
	    }
	}
	if( $current->{fontname} != $next->{fontname} ||
	   $current->{fontsize} != $next->{fontsize} ) {
	    if( $next->{fontname} ne '' && $next->{fontsize} > 0 ) {
		$graphic->printFont( );
	    }
	    $current->{fontname} = $next->{fontname};
	    $current->{fontsize} = $next->{fontsize};
	}
	$next->{TChange} = 0;
    }
}

# }}}
# {{{ Methods to access and set values in gstate objects

sub setGstate {
    my $self = shift;
    my($attr,$val) = @_;
    $self->{GChange} = 1 if $val ne $self->{gstate}{$attr};
    $self->{gstate}{$attr} = $val;
}

sub getGstate {
    my $self = shift;
    $self->{gstate}{$_[0]};
}

sub setTstate {
    my $self = shift;
    my($attr,$val) = @_;
    $self->{TChange} = 1 if $val ne $self->{tstate}{$attr};
    $self->{tstate}{$attr} = $val;
}

sub getTstate {
    my $self = shift;
    $self->{tstate}{$_[0]};
}

sub setFontName {
    my $self = shift;
    $self->{TChange} = 1 if $_[0] ne $self->{fontname} ;
    $self->{fontname} = $_[0];
}

sub getFontName {
    my $self = shift;
    $self->{fontname};
}

sub setFontSize {
    my $self = shift;
    $self->{TChange} = 1 if $_[0] ne $self->{fontsize} ;
    $self->{fontsize} = $_[0];
}

sub getFontSize {
    my $self = shift;
    $self->{fontsize};
}

# }}}
# {{{ Set Color parameters

# Access color name, colorspace and color value. If fourth parameter
# is defined then set name, colorspace or color value. For
# setColorName and setColorSpace checks are done to see whether to
# set $cstate_change. This check is not done for setColorValue because
# it is a more costly operation, and is left to gstate->update to
# check.

sub setColorName {
    my $self = shift;
    my($sf,$val) = @_;
    $self->{CChange} = 1 if $val ne $self->{$sf}{name};
    $self->{$sf}{name} = $val;
}

sub getColorName {
    my $self = shift;
    my $sf = shift;
    $self->{$sf}{name};
}

sub setColorSpace {
    my $self = shift;
    my($sf,$val) = @_;
    $self->{CChange} = 1 if $val ne $self->{$sf}{cspace};
    $self->{$sf}{cspace} = $val;
}

sub getColorSpace {
    my $self = shift;
    my $sf = shift;
    $self->{$sf}{cspace};
}

# These routines take and return references to arrays

sub setColorValue {
    my $self = shift;
    my($sf,$valref) = @_;
    ref($valref) eq 'ARRAY' || message_err( 'Internal - parameter must be an ARRAY ref' );
    $self->{CChange} = 1;
    $self->{$sf}{value} = [ @$valref ];
}

sub getColorValue {
    my $self = shift;
    my $sf = shift;
    $self->{$sf}{value};
}

# }}}
# {{{ Miscellaneous

# Forces the change variables (used by gstate->update).

sub change {
    my $self = shift;
    my(@p) = @_;

    foreach (@p) {
	$self->{GChange} = 1 if( m/^g/ );
	$self->{CChange} = 1 if( m/^c/ );
	$self->{TChange} = 1 if( m/^t/ );
    }
}

# Static methods. Verify that floats are equal within a specific
# margin. Useful for matching two equal colors, for example [ 1, 0,
# 0.5200 ] and [ 1, 0, 0.52008 ].

BEGIN {
    $Pdf::Gstate::float_equal_margin = 0.0001;
}

sub arrayEqual {
    my( $aptr, $bptr ) = @_;
    my(@a) = @{$aptr};
    my(@b) = @{$bptr};
    return 0 if( (@a) != (@b) );
    my $i;
    for( $i=0; $i<scalar(@a); ++$i ) {
	# jpravetz 9apr96 - fixed this line
	return 0 if( !floatEqual($a[$i],$b[$i]) );
    }
    1;
}

sub floatEqual {
    my($a,$b) = @_;
    if( abs( $a - $b ) < $Pdf::Gstate::float_equal_margin ) {
	1;
    } else {
	0;
    }
}

# Debug routine. Really useful with Perl5 debugger.

sub printState {
    my($self) = shift;
    my $comment = shift;

    print( STDERR "-----$comment-----\ngstate:",
	  ' w=', $self->{gstate}{w},
	  ', i=', $self->{gstate}{i},
	  ', j=', $self->{gstate}{j},
	  ', J=', $self->{gstate}{J},
	  ', d=', $self->{gstate}{d},
	  ', M=', $self->{gstate}{M}, "\n" );

    my $k;
    foreach $k ('stroke','fill') {
	print( STDERR $k,
	      ': name=', $self->{$k}{name},
	      ', cspace=', $self->{$k}{cspace},
	      ', value=[', join(',',@{$self->{$k}{value}}), "]\n" );
    }

    print( STDERR "tstate:",
	  ' i=', $self->{gstate}{i},
	  ', Tc=', $self->{tstate}{Tc},
	  ', Tw=', $self->{tstate}{Tw}.
	  ', Tz=', $self->{tstate}{Tz},
	  ', TL=', $self->{tstate}{TL},
	  ', Ts=', $self->{tstate}{Ts},
	  ', Tr=', $self->{tstate}{Tr}, "\n" );

    print( STDERR "tstate:",
	  ' fontname=', $self->{fontname},
	  ', fontsize=', $self->{fontsize}, "\n" );

    print( STDERR "Change:",
	  ' G=', $self->{GChange},
	  ' C=', $self->{CChange},
	  ' T=', $self->{TChange}, "\n" );
}


# }}}
# {{{ Emacs local variables  

### Local variables:
### major-mode: perl-mode
### perl-indent-level: 4
### voice-lock-mode: t
### folded-file: t
### End:

# }}}
1;
