# $Id: Obj.pm,v 1.30 1996/06/19 18:42:18 raman Exp $
require 5.000;
use Pdf::Std qw( message_err pdfDateString );
use strict qw ( subs vars refs );

package Pdf::Obj;
# {{{  Pdf::Obj

# {{{  commentary:

#############################################################################
#Indirect objects in a PDF file:
#raman: the reference to 'stream' in the comment below is questionable.
# This package is used to generate PDF Objects. These are objects that
# exist in the PDF file as entries in the XREF table. A PDF object is
# composed of a number of flags, plus a reference to Contents and
# Stream. The Contents is usually a reference to a COS Dictionary, but
# can be any COS object. Do not confuse a PdfObj with COS
# objects, which are building blocks in PDF. Routines in Pdf::Obj are
# usually inherited by more explicitly named PDF Objects
# (eg. Pdf::Info).

# }}}

# Create a new PDF object. This is most often called from within a new
# method of an overriding class.
#Passing in a list of arguments results  in the newly created object
#having a dictionary containing these. (raman: why-- questionable)
#raman: grep on *.pm shows that no one else uses newObjDict --may be safe to
# nuke it.

sub new {
    my $class = shift;
    my $self = {};
    bless $self, $class;
    $self->newObjDict( @_ ) if @_;
    $self;
}

# Use refType instead of using ref() on the object, for code
# portability. Use the syntax $reftype = eval { $objref->refType },
# otherwise a runtime error will occur if $objref does not have a
# refType method.

sub refType {
    'PdfObj';
}

# Access generic top-level flags by flag name/key

sub setFlag {
    my $self = shift;
    #raman: Modified @_ to $_ on next line as per perl -cw
    $self->{$_[0]} = $_[1];
}

sub getFlag {
    my $self = shift;
    $self->{$_[0]}
    or die "Flag $_[0] not defined in $self";
}

# Access the assigned XREF object number of the PDF object

sub setObjNum {
    my $self = shift;
    $self->{'Obj'} = shift;
}

sub getObjNum {
    my $self = shift;
    $self->{'Obj'};
    #raman: removed the "die" here since other code relies on the
    #undef --specifically,
    #look at method printObj for class Obj. it uses lateObj if number
    #is not defined
}

# Access the print flag of the PDF object. If this flag is 1 then this
# object has already been written to the output stream.

sub setPrint {
    my $self = shift;
    $self->{'Print'} = 1;
}

sub clearPrint {
    my $self = shift;
    $self->{'Print'} = 0;
}

sub getPrint {
    my $self = shift;
    ( $self->{'Print'} == 1 ) ? 1 : 0 ;
}


# These routines apply when the contents of a Pdf Object is a Cos Dict
# (this is almost always the case).  They provide access to the top
# level dict in the pdf object, contained in the Contents field.
#raman: these are basically short-cut subroutines and may be well left out--

sub newObjDict {
    my $self = shift;
    $self->{'Contents'} = Pdf::CosDict->new( @_ );
}

sub setObjDict {
    my $self = shift;
     $self->{'Contents'} = Pdf::CosDict->new()
      unless defined ($self->{'Contents'});
    $self->{'Contents'}->set( @_ );
}

sub getObjDict {
    my $self = shift;
    my $key = shift;
    die "Object $self does not have a contents field"
      unless defined ($self->{'Contents'});
    $self->{'Contents'}->get( $key );
}

# Returns a reference to the Dictionary COS object
#raman: this sub's name is counter-intuitive. It returns a perl5 ref
#not a "1 0 R" as in PDF
#Is better to rename it  getObjDictContents (if we keep it at all)

sub getObjDictRef {
    my $self = shift;
    $self->{'Contents'};
}

# Only use when @_[0] is the key to an array

sub pushObjDictArray {
    my $self = shift;
    $self->{'Contents'}->pushArray( @_ );
}

# This routine sets the Pdf Object Contents field to whatever, where
# whatever is expected to be a reference to a Cos Object.

sub setObjContents {
    my $self = shift;
    my $cosref = shift;
    #raman: see if eval can be avoided.
    eval{ $cosref->refType } =~ /^Cos/ ||
	message_err( 'Pdf Obj Contents must be a reference to a Cos Object' );
    $self->{'Contents'} = $cosref;
}

sub getObjContents {
    my $self = shift;
    $self->{'Contents'};
}

# Standard printObj routine. This routine is almost always inherited
# by an overriding class.

sub printObj {
    my $self = shift;
    my $file = shift;

    return if $self->getPrint;

    eval{ $file->refType } eq 'File' || message_err( 'Requires reference to File object' );
    if( $self->getObjNum ) {
	$file->lateObj( $self->getObjNum() );
    } else {
	$self->setObjNum( $file->beginObj() );
    }

    if( defined( $self->{'Contents'} ) ) {
	$self->{'Contents'}->printObj( $file );
	$file->pdfPrint( "\n" );
    }
    if( $self->{'Buffer'} == 1 && ref( $self->{'Stream'} ) ) {
	$self->{'Stream'}->printObj( $file );
    }
    
    $file->endObj;
    $self->setPrint;
}

# }}}
# {{{ commentary

# The remainder of this file defines COS object types. COS objects are
# used as building blocks to make PDF objects. The COS object types
# used here correspond closely, but not exactly, with the COS objects
# in Adobe Acrobat. The word 'COS' has historical meaning at
# Adobe. For the purposes of this distribution 'Cos' can be thought of
# as 'basic PDF object building block'.

# }}}
# {{{ Simple COS objects

package Pdf::CosString;

sub new {
    my $class = shift;
    my $self = shift;
    bless \$self, $class;
}

sub refType {
    'CosString';
}

sub set {
    my $self = shift;
    $$self = shift;
}

sub get {
    my $self = shift;
    $$self;
}

sub printObj {
    my $self = shift;
    my $file = shift;
    $file->pdfPrint( '(', $$self, ')' );
}
    
package Pdf::CosInteger;
@Pdf::CosInteger::ISA = qw( Pdf::CosString );

sub refType {
    'CosInteger';
}

sub printObj {
    my $self = shift;
    my $file = shift;
    $file->pdfPrint( $$self );
}
    
package Pdf::CosName;
@Pdf::CosName::ISA = qw( Pdf::CosString );

sub refType {
    'CosName';
}

sub printObj {
    my $self = shift;
    my $file = shift;
    $file->pdfPrint( "/$$self" );
}
    
package Pdf::CosNull;

sub new {
    my $class = shift;
    my $self = 'null';		# value not actually used
    bless \$self,$class;
}

sub refType {
    'CosNull';
}

sub printObj {
    my $self = shift;
    my $file = shift;
    $file->pdfPrint( 'null' );
}
    
package Pdf::CosBoolean;
@Pdf::CosBoolean::ISA = qw( Pdf::CosString );

sub refType {
    'CosBoolean';
}

sub new {
    my $class = shift;
    my $self = shift;
    if( $self == 1 || $self =~ /^t/i ) {
	$self = 'true';
    } else {
	$self = 'false';
    }
    bless \$self, $class;
}

sub set {
    my $self = shift;
    my $val = shift;
    if( $val == 1 || $val =~ /^t/i ) {
	$$self = 'true';
    } else {
	$$self = 'false';
    }
}

sub printObj {
    my $self = shift;
    my $file = shift;
    $file->pdfPrint( $$self );
}

# }}}
# {{{ Special Cos Objects

# Indirect reference to a pdf object

package Pdf::CosRef;
#Warning--Overloading  of term 'reference' in comment
# Make a (perl5) reference to a Pdf Object reference. This is used whenever an
# indirect object reference exists within a PDF Object.

sub new {
    my $class = shift;
    my $self = shift;
    ( eval{ $self->refType } =~ /^Pdf/  ) ||
	message_err( 'CosRef must be a reference to a PDF Object' );
    bless \$self, $class;
}

sub refType {
    'CosRef';
}

sub getObjNum {
    my $self = shift;
    ${$self}->getObjNum();
}

# Dereference the reference to get the actual PDF Object reference

sub getObjRef {
    my $self = shift;
    ${$self};
}

sub printObj {
    my $self = shift;
    my $file = shift;
    defined( ${$self}->getObjNum ) ||
	message_err( "No object number assigned for referenced object" );
    $file->pdfPrint( ${$self}->getObjNum, " 0 R" );
}

# }}}
# {{{ Cos Streams, Arrays and Dictionaries

package Pdf::CosStream;
#raman: Warning-- this implementation does not seem to have room for the
#stream dictionary

sub new {
    my $class = shift;
    my $self = '';
    bless \$self, $class;
}

sub refType {
    'CosStream';
}
#Raman: This next sub does not do what the name implies

sub printStream {
    my $self = shift;
    $$self .= @_;
}

sub printObj {
    my $self = shift;
    my $file = shift;

    $file->pdfPrint( "\nstream\n" );
    $file->pdfPrint( $$self );
    $file->pdfPrint( "endstream\n" );
}

package Pdf::CosArray;

sub new {
    my $class = shift;
    my $self = [ @_ ];
    bless $self, $class;
}

sub refType {
    'CosArray';
}

# Returns array

sub get {
    my $self = shift;
    @$self;
}

sub push {
    my $self = shift;
    push( @$self, @_ );
}

sub printObj {
    my $self = shift;
    my $file = shift;

    $file->pdfPrint( '[' );
    my $i;
    foreach $i (0..$#{ $self }) {
	$self->[$i]->printObj( $file );
	$file->pdfPrint( ($i==$#{$self})?'':' ' );
    }
    $file->pdfPrint( "]" );
}

# Shortcut to make a list of references to numbers.
#raman: This sub's name does not imply what it  actually does

sub newInteger {
    my $class = shift;
    my $self = [];
    my $entry;
    foreach $entry (@_) {
	push( @$self, Pdf::CosInteger->new( $entry ) );
    }
    bless $self,$class;
}

# }}}
# {{{ package Pdf::CosDict

package Pdf::CosDict;

sub new {
    my $class = shift;
    my $self = {};
    bless $self, $class;
    $self->setOrder( @_ ) if @_;
    $self;
}

sub refType {
    'CosDict';
}

sub setOrder {
    my $self = shift;
    $self->{'Order'} = [ @_];
}

sub set {
    my $self = shift;
    my(%p) = @_;
    my $key;
    foreach $key (keys %p) {
	$self->{'Hash'}{$key} = $p{$key};
    }
}

sub get {
    my $self = shift;
    my $key = shift;
    $self->{'Hash'}{$key};
}

# Use only when Dict entry is a reference to an array

sub pushArray {
    my $self = shift;
    my( $key, @val ) = @_;
    if( eval{ $self->{'Hash'}{$key}->refType } ne 'CosArray' ) {
	$self->{'Hash'}{$key} = Pdf::CosArray->new( @val );
    } else {
	push( @{$self->{'Hash'}{$key}}, @val );
    }
}

# Print the Dictionary. If Order is not defined or is an empty list
# then print all entries in the Hash table.

sub printObj {
    my $self = shift;
    my $file = shift;

    $file->pdfPrint( '<<' );
    my($attr,$val);
    my @order;
    if( defined( $self->{'Order'} ) && scalar(@{$self->{'Order'}}) > 0 ) {
	@order = @{$self->{'Order'}};
    } else {
	@order = keys %{$self->{'Hash'}};
    }
    foreach $attr ( @order ) {
	$val = $self->{'Hash'}{$attr};
	if( defined( $val ) ) {
	    $file->pdfPrint( "/$attr " );
	    $val->printObj( $file );
	    $file->pdfPrint( "\n" );
	}
    }
    $file->pdfPrint( ">>" );
}

# }}}
# {{{ Emacs local variables  

### Local variables:
### major-mode: perl-mode
### perl-indent-level: 4
### voice-lock-mode: t
### folded-file: t
### End:

# }}}
1;
