# $Id: Page.pm,v 1.19 1996/06/12 00:24:30 raman Exp $
package Pdf::Page;
require 5.000;
use Pdf::Obj;
use Pdf::Pages;
use Pdf::Std qw( message_err pdfEscape f0 f2 f3 f4 );
use strict qw ( subs vars refs );
@Pdf::Page::ISA = qw( Pdf::Obj );

# {{{ Page Operations

BEGIN {
    $Pdf::Page::gstack_limit = 10; # max depth of gsave stack
}

sub new {
    my $class = shift;
    my $fileref = shift;
    my $pagenum = shift;
    eval{ $fileref->refType } eq 'File' ||
	message_err( 'Parameter is not a reference to a Pdf::File' );
    defined ($pagenum && $pagenum =~ /^\d+$/ ) 
|| message_err( 'No page number' );

    my $self = Pdf::Obj->new( 'Type', 'Parent', 'Resources', 'MediaBox', 
			     'CropBox', 'Rotate', 'Contents', 'Annots',
			     'B', 'Thumb', 'Dur', 'Hid', 'Trans' );
    $self->setFlag( 'Number', $pagenum );
    $self->setFlag( 'File', $fileref );
    $self->setObjDict( 'Type' => Pdf::CosName->new( 'Page' ) );
    bless $self,$class;
}

# Initialise Graphics State. This must be done if the page contains
# any marks whatsoever. Graphics.pm must also be loaded in this event.

sub initGraphic {
    my $self = shift;
    my $file = shift;

    if( eval{ $file->existGraphic } ) {
	$self->{GCurrent} = Pdf::Gstate->new;
	$self->{GCurrent}->init;
	$self->{GNext} = Pdf::Gstate->new;
	$self->{GNext}->init;
	$self->{Procset}{PDF} = 1;
	1;
    } else {
	0;
    }
}

sub refType {
    'PdfPage';
}



sub validate {
    my $self = shift;
    # May need to validate that there is a Parent, but not likely
    # since it is set automatically and not by the user
}
# Allows user to set certain page attributes
# My short-term way of sharing some code with Pages.pm. I may move to
# inherit methods from Pages.pm if this does not work, or if more code
# can be shared.

sub setDict {
    my $self = shift;
    $self->Pdf::Pages::setDict( @_ );
}

# }}}
# {{{ Close a Page

# Optional method to execute when done with a page. If there are many
# pages in the document then this will save on memory: does DESTROY
# Gstate, which is no longer needed. If passed a parameter 'resource'
# then a resource dictionary is built for the page.

# Not tested.

sub closePage {
    my $self = shift;
    $self->setFlag( 'Done', 1 );
#nuke the graphic states
    $self->{GCurrent}->DESTROY if defined( $self->{GCurrent} );
    $self->{GNext}->DESTROY if defined( $self->{GNext} );

    # Build a Resource Dictionary 
    
    if( shift =~ /^resource/i ) {
	$self->buildResource();
    }
}

# Build a Resource Dictionary and attach it to $self (Resource is an
# enclosed object, not an indirect object).

sub buildResource {
    my $self = shift;

    my $resource = Pdf::CosDict->new( 'ProcSet', 'Font', 'XObject' );

    my($key,$ref);
    my $dict;

    # Process ProcSets
#raman: procsets hardwired here     
    foreach $key ( 'PDF', 'Text', 'ImageB', 'ImageC', 'ImageI' ) {
	if( $self->{ProcSet}{$key} == 1 ) {
	    $resource->pushArray( 'ProcSet', Pdf::CosName->new( $key ) );
	}
    }

    # Process Fonts

    if( scalar(keys %{$self->{Font}}) ) {
	$dict = Pdf::CosDict->new;
	foreach $key (keys %{$self->{Font}}) {
	    $key =~ /^F\d+$/
 || message_err( "Font '$key' is not in form 'F1'" );
	    $ref = $self->{Font}{$key};
	    eval{ $ref->refType } eq 'PdfFont' ||
		message_err( 'Font Resource requires a PdfFont reference' );
	    $dict->set( $key, Pdf::CosRef->new( $ref ) );
	}
	$resource->set( 'Font', $dict ); 
    }

    # Process XObjects

    if( scalar(keys %{$self->{XObject}}) ) {
	my $dict = Pdf::CosDict->new;
	foreach $key (keys %{$self->{XObject}}) {
	    $key =~ /^Im\d+$/ || message_err( "XObject '$key' is not in form 'F1'" );
	    $ref = $self->{XObject}{$key};
	    eval{ $ref->refType } eq 'PdfXObject' ||
		message_err( 'XObject Resource requires a PdfXObject reference' );
	    $dict->set( $key, Pdf::CosRef->new( $ref ) );
	}
	$resource->set( 'XObject', $dict ); 
    }

    
    $self->setObjDict( 'Resources', $resource );

}

# }}}
# {{{ Set objects and attributes

# Methods allow particular objects and attributes to be added to an
# initialised page.

sub setParent {
    my $self = shift;
    my $objref = shift;
    eval{ $objref->refType } eq 'PdfPages' ||
	message_err( 'Page Parent must be a PdfPages object' );
    $self->setObjDict( 'Parent', Pdf::CosRef->new( $objref ) );
}

sub addContent {
    my $self = shift;
    my $objref = shift;
    eval{ $objref->refType } eq 'PdfGraphic' ||
	message_err( 'Reference to PdfGraphic required' );
    $self->pushObjDictArray( 'Contents',  Pdf::CosRef->new( $objref ) );
}
# {{{ subs that are nuked for now:

# sub addAnnot {
#     my $self = shift;
#     my $objref = shift;
#     eval{ $objref->refType } =~ /^PdfAnnot/ ||
# 	message_err( 'Reference to PdfAnnot required' );
#     $self->pushObjDictArray( 'Annots',  Pdf::CosRef->new( $objref ) );
# }

# sub addBead {
#     my $self = shift;
#     my $objref = shift;
#     eval{ $objref->refType } =~ /^PdfBead/ ||
# 	message_err( 'Reference to PdfFead required' );
#     $self->pushObjDictArray( 'B',  Pdf::CosRef->new( $objref ) );
# }

# }}}

# }}}
# {{{ Resource Flags

# Flags are set to indicate which resources are use. Resource
# dictionary is built by buildResource.

# $name must be one of 'PDF', 'Text', 'ImageB', 'ImageC', 'ImageI'

sub useProcSet {
    my $self = shift;
    my $name = shift;
    $self->{ProcSet}{$name} = 1;
}

# Adds {Font}{F1} = $fontref to PdfPage object. $name must be in form
# 'Helvetica'.

sub useFont {
    my $self = shift;
    my $name = shift;
    defined( my $fontref = $self->{File}->getFontRef( $name ) )
 || message_err( "Font '$name' not registered for use" );
    $self->{Font}{$fontref->getName()} = $fontref;
}

# Adds XObject.

sub useXObject {
    my $self = shift;
    my $name = shift;
    message_err( 'XObjects not yet supported' );
}

# }}}
# {{{ gsave grestore

# Save gstate on internally maintained stack. These should only be
# maintained if Gstate.pm is loaded, and if initGraphic has been
# executed on the page.

sub gsave {
    my $self = shift;
    
    # Increment state pointer

    if( !defined($self->{GStackIndex}) ) {
	$self->{GStateIndex} = 0;
        $self->{GStackIndex} = 0;
    } elsif( $self->{GStackIndex} > $Pdf::Page::gstack_limit ) {
	message_err( "Gstate stack level exceeds $Pdf::Page::gstack_limit" );
    }

    # Store gstate onto stack 
    if (defined ($self->{GCurrent})) {
    $self->{GStack}{$self->{GStackIndex}} = $self->{GCurrent}->copy;
} else {
    warn "gsave:Could not copy GCurrent since it was not defined
gstate stack level is ", $self->{GStackIndex};
}
    $self->{GStackIndex}++;
}

sub grestore {
    my $self = shift;

    if( $self->{GStackIndex} < 0 ) {
	message_err( "ERROR: Tried to do grestore off empty stack" );
    }

    # Set GCurrent to point to bottom object of stack. Copy GCurrent
    # to CNext.

    $self->{GCurrent} = $self->{GStack}{$self->{GStackIndex}};
    if (defined ($self->{GCurrent})) {
    $self->{GNext} = $self->{GCurrent}->copy( $self->{GNext} );
} else {
    warn "grestore: could not copy GCurrent since it is not defined here
gstack level is ", $self->{GStackIndex};
}

    # Decement stack pointer

    --$self->{GStackIndex};
}

# }}}
# {{{ package Pdf::File

package Pdf::File;

# Must be called once for every page. Consider using the optional
# method closePage after you are done writting to the page.

sub newPage {
    my $self = shift;
    my $pageref = $self->newEmptyPage( @_ );
    $pageref->initGraphic( $self );
    return $pageref;
}

# Creates a new page, but does not initialise graphics state.

sub newEmptyPage {
    my $self = shift;

    # If the first parameter is a number then use that as the page number,
    # otherwise increment and use $self->{PageIndex}.

    my $pagenum;
    if( $_[0] =~ /^\d+$/ ) {
	$pagenum = shift;
	$self->{PageIndex} = $pagenum;
    } else {
	$pagenum = ++$self->{PageIndex};
    }
    
    my %p = @_;

    # Ensure that the page has not previously been defined

    !defined( $self->{Page}{$pagenum} ) ||
	message_err( "Page $pagenum already defined" );

    my $pageref = Pdf::Page->new( $self, $pagenum );
    $pageref->setDict(%p);

    $self->setPageRef( $pagenum, $pageref );
    $self->addObjRef( $pageref );

    # Return page number and reference to page object

    ( $pagenum, $pageref );
}

# }}}
# {{{ Emacs local variables  

### Local variables:
### major-mode: perl-mode
### perl-indent-level: 4
### voice-lock-mode: t
### folded-file: t
### End:

# }}}
1;
