#$Id: Render.pm,v 1.14 1996/06/17 23:12:11 raman Exp $
package Pdf::Render;
package Pdf::Graphic;
require 5.000;
use Pdf::Std qw( assertFloat assertFloatRange message_err
		f0 f1 f2 f3 f4 af0 af1 af2 af3 af4 pdfEscape );
use strict qw ( subs vars refs );
@Pdf::Graphic::ISA = qw( Pdf::Obj );

# {{{  commentary

# This file is part of Graphic.pm. It should just be included via
# 'require', but there are some path problems doing this. Eventually I
# will tag this on to the end of Graphic.pm, or find another way to
# 'require' it.

# }}}
# {{{ Graphics state print routines

# Print methods used by Pdf::Gstate::update, and also by methods in
# the stream package that force graphics state.

sub printGstate {
    my $self = shift;
    my $k = shift;
    $self->{path} == 0 || message_err( '' );
    $self->strPrint( f3($self->{GNext}->{gstate}{$k}), " $k\n" );
}

sub printGstateD {
    my $self = shift;
    $self->strPrint( $self->{GNext}->{gstate}{d}, " d\n" );
}

sub printTstate {
    my $self = shift;
    my $k = shift;
    $self->textPrint( f3($self->{GNext}->{tstate}{$k}), " $k\n" );
}

sub printFont {
    my $self = shift;
    my $name = $self->{File}->getFontRef( $self->{GNext}->{fontname} )->getName();
    $self->textPrint( '/', $name, ' ',
		     f3( $self->{GNext}->{fontsize} ), " Tf\n" );
}

BEGIN {  # Defines the operators used to change colors

    %Pdf::Stream::colorop = (
		'stroke', {
		    'DeviceGray' => 'G',
		    'DeviceRGB' => 'RG',
		    'DeviceCMYK' => 'K',
		    'CalGray' => 'SC',
		    'CalRGB' => 'SC',
		    'CalCMYK' => 'SC',
		    'Lab' => 'SC',
		    'Indexed' => 'SC' },
		'fill', {
		    'DeviceGray' => 'g',
		    'DeviceRGB' => 'rg',
		    'DeviceCMYK' => 'k',
		    'CalGray' => 'sc',
		    'CalRGB' => 'sc',
		    'CalCMYK' => 'sc',
		    'Lab' => 'sc',
		    'Indexed' => 'sc' }
		);

    # Defines the operators used to change the colorspace

    %Pdf::Stream::colorspaceop = ( 'stroke' => 'CS',
				  'fill' => 'cs' );
}

# Writes color and colorspace changes to file. This is executed once
# it has been determined that a color change has occured and we want
# to write the values. Also copies color values in $next to $current.

sub printColor {
    my $self = shift;
    my $sf = shift;

    my $next = $self->{GNext};
    my $current = $self->{GCurrent};

    # If this was a predefined name then we only have a name to work with,
    # so we have to fill in the rest of the fields (value array and colorspace).
    # We do not have to do this anymore - it is done by setColor
    #     if( defined( $Pdf::Base::color_space{$nref->{name}} ) ) {
    # 	$nref->{cspace} = $Pdf::Base::color_space{$nref->{name}};
    # 	@{$nref->{value}} = @{$Pdf::Base::color_val{$nref->{name}}};
    #     }
	
    my $newcs = $next->getColorSpace( $sf );
    my $oldcs = $current->getColorSpace( $sf );

    # Write any change of color or colorspace to pdf file

    if( $newcs =~ /^Device/ ) {
	$self->anyPrint( join( ' ', af4(@{$next->getColorValue( $sf )}),
				     $Pdf::Stream::colorop{$sf}->{$newcs} ), "\n" );

    } elsif( $newcs =~ /^CS\d/ ) {
	my $calnewcs = $self->{File}->{ColorSpace}{$newcs}->get( 'ColorSpace' );
	if( $newcs ne $oldcs ) {
	    $self->anyPrint( "/$newcs $Pdf::Stream::colorspaceop{$sf}\n" );
	}
	$self->anyPrint( join(' ', af4(@{$next->getColorValue( $sf )}),
				     $Pdf::Stream::colorop{$sf}->{$calnewcs} ), "\n" );

    } else {
	message_err( "Unrecognised colorspace '$newcs'" );
    }
	
    # Now set current values to next values in gstate object

    $current->setColorName( $sf, $next->getColorName( $sf ) );
    $current->setColorSpace( $sf, $next->getColorSpace( $sf ) );
    $current->setColorValue( $sf, $next->getColorValue( $sf ) );
}

# }}}
# {{{ Primitive Path Operators
    
sub moveto {
    my $self = shift;
    assertFloat(@_) == 2 || message_err( 'moveto requires 2 numbers' );
    $self->setPath;
    $self->strPrint( join( ' ', af3(@_), "m\n" ) );
}


sub lineto {
    my $self = shift;
    assertFloat(@_) == 2 || message_err( 'lineto requires 2 numbers' );
    $self->setPath;
    $self->strPrint( join( ' ', af3(@_), "l\n" ) );
}


sub curveto {
    my $self = shift;
    assertFloat(@_) == 6 || message_err( "curveto requires 6 numbers" );
    $self->setPath;
    $self->strPrint( join( ' ', af3(@_) ), " c\n" );
}

sub vCurveto {
    my $self = shift;
    assertFloat(@_) == 6 || message_err( "vCurveto requires 6 numbers" );
    $self->setPath;
    $self->strPrint( join( ' ', af3(@_) ), " v\n" );
}

sub yCurveto {
    my $self = shift;
    assertFloat(@_) == 4 || message_err( "yCurveto requires 4 numbers" );
    $self->setPath;
    $self->strPrint( join( ' ', af3(@_) ), " y\n" );
}

sub rectangle {
    my $self = shift;
    assertFloat(@_) == 4 || message_err( "rectangle requires 4 numbers" );
    $self->clearPath;
    $self->strPrint( join( ' ', af3(@_) ), " re\n" );
}

sub closepath {
    my $self = shift;
    $self->clearPath;
    $self->strPrint( "h\n" );
}

sub endpath {
    my $self = shift;
    $self->clearPath;
    $self->strPrint( "n\n" );
}

# Clipping operators (do not require gstate to be updated)

sub clip {
    my $self = shift;
    $self->getPath && message_err( 'Cannot do clip within a path' );
    $self->strPrint( "W\n" );
}

sub eoclip {
    my $self = shift;
    $self->getPath && message_err( 'Cannot do clip within a path' );
    $self->strPrint( "W*\n" );
}

# }}}
# {{{ Primitive Graphics State Commands

# Local helper methods

sub forceGstate {
    my $self = shift;
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->{GNext}->setGstate( @_ );
    $self->{GCurrent}->setGstate( @_ );
    $self->printGstate( $_[0] );
}

# Sets the appropriate attribute in the next gstate object. For set
# methods changes will be implemented at the next start of path or
# stroke/fill operator. For force methods changes will be make
# immediately, and regardless of current and next gstate.

sub setLinewidth {
    my $self = shift;
    assertFloat( @_ ) == 1 || message_err( 'not a number' );
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->{GNext}->setGstate( 'w', @_ );
}

sub forceLinewidth {
    my $self = shift;
    assertFloat( @_ ) == 1 || message_err( 'not a number' );
    $self->forceGstate( 'w', @_ );
}

sub setFlatness {
    my $self = shift;
    assertFloat( @_ ) == 1 || message_err( 'not a number' );
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->{GNext}->setGstate( 'i', @_ );
}

sub forceFlatness {
    my $self = shift;
    assertFloat( @_ ) == 1 || message_err( 'not a number' );
    $self->forceGstate( 'i', @_ );
}

sub setLineCapStyle {
    my $self = shift;
    assertFloat( @_ ) == 1 || message_err( 'not a number' );
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->{GNext}->setGstate( 'J', @_ );
}

sub forceLineCapStyle {
    my $self = shift;
    assertFloat( @_ ) == 1 || message_err( 'not a number' );
    $self->forceGstate( 'J', @_ );
}

sub setJoinStyle {
    my $self = shift;
    assertFloat( @_ ) == 1 || message_err( 'not a number' );
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->{GNext}->setGstate( 'j', @_ );
}

sub forceJoinStyle {
    my $self = shift;
    assertFloat( @_ ) == 1 || message_err( 'not a number' );
    $self->forceGstate( 'j', @_ );
}

sub setMiterLimit {
    my $self = shift;
    assertFloat( @_ ) == 1 || message_err( 'not a number' );
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->{GNext}->setGstate( 'M', @_ );
}

sub forceMiterLimit {
    my $self = shift;
    assertFloat( @_ ) == 1 || message_err( 'not a number' );
    $self->forceGstate( 'M', @_ );
}

# Calling syntax is $graphic->setDashPattern( [ 1, 3 ], 3 );

sub setDashPattern {
    my $self = shift;
    my(@p) = @_;
    scalar(@p) == 2 || message_err( 'Requires 2 elements' );
    assertFloat( @p[1] ) == 1 || message_err( '2nd parameter not a number' );
    ref(@p[0]) eq 'ARRAY'  || message_err( '1st parameter not an ARRAY' );
    assertFloat( @{$p[0]} ) >= 1 || message_err( '1st parameter not an array of numbers' );
    # $self->{GNext}->setGstate( 'd', "@{$p[0]} $p[1]" );   # jpravetz 08-apr-1996
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->{GNext}->setGstate( 'd', '[' . join(' ',af3(@{$p[0]})) . '] ' . f3($p[1]) );
}

sub forceDashPattern {
    my $self = shift;
    my(@p) = @_;
    scalar(@p) == 2 || message_err( 'Requires 2 elements' );
    assertFloat( @p[1] ) == 1 || message_err( '2nd parameter not a number' );
    ref(@p[0]) eq 'ARRAY'  || message_err( '1st parameter not an ARRAY' );
    assertFloat( @{$p[0]} ) >= 1 || message_err( '1st parameter not an array of numbers' );
    # my $val = "\[@{$p[0]}\] $p[1]";   # jpravetz 08-apr-1996
    my $val = '[' . join(' ',af3(@{$p[0]})) . '] ' . f3($p[1]);
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->{GNext}->setGstate( 'd', $val );
    $self->{GCurrent}->setGstate( 'd', $val );
    $self->printGstateD();
    # $self->textPrint( "$val d\n" );
}

# }}}
# {{{ gupdate, gclear, gsave, grestore, and ctm

sub gupdate {
    my $self = shift;
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->{GNext}->update( $self->{GCurrent}, $self, @_ );
}

sub gclear {
    my $self = shift;
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->{GNext}->init;
}

sub gsave {
    my $self = shift;
    $self->gupdate();
    $self->getPageRef->gsave;
    $self->strPrint( "q\n" );
}

sub grestore {
    my $self = shift;
    $self->getPageRef->grestore;
    $self->strPrint( "Q\n" );
}

sub ctm {
    my $self = shift;
    ( assertFloat(@_) == 6 ) || message_err( 'ctm requires 6 numbers' );
    # This next line is done in gupdate() - but I leave it here as a reminder
    # $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->gupdate();
    $self->strPrint( join( ' ', af4(@_), "cm\n" ) );
}

# Whenever transitioning into a 'within path' condition, this routine
# is called to potentially update gstate.

sub setPath {
    my $self = shift;
    # Only do gupdate on first entry into path
    $self->gupdate() if $self->{Path} == 0;
    $self->{Path} = 1;
}

sub getPath {
    my $self = shift;
    $self->{Path};
}

sub clearPath {
    my $self = shift;
    $self->{Path} = 0;
}

# }}}
# {{{ Primitive Mark Operators

# Mark operators force update of gstate, but not text state.

sub stroke {
    my $self = shift;
    $self->clearPath;
    $self->gupdate( 'notstate' );
    $self->strPrint( "S\n" );
}

sub closepathStroke {
    my $self = shift;
    $self->clearPath;
    $self->gupdate( 'notstate' );
    $self->strPrint( "s\n" );
}

sub fill {
    my $self = shift;
    $self->clearPath;
    $self->gupdate( 'notstate' );
    $self->strPrint( "f\n" );
}

sub evenOddFill {
    my $self = shift;
    $self->clearPath;
    $self->gupdate( 'notstate' );
    $self->strPrint( "f*\n" );
}

sub fillStroke {
    my $self = shift;
    $self->clearPath;
    $self->gupdate( 'notstate' );
    $self->strPrint( "B\n" );
}

sub closepathFillStroke {
    my $self = shift;
    $self->clearPath;
    $self->gupdate( 'notstate' );
    $self->strPrint( "b\n" );
}

sub evenOddFillStroke {
    my $self = shift;
    $self->clearPath;
    $self->gupdate( 'notstate' );
    $self->strPrint( "B*\n" );
}

# }}}
# {{{ Complex Path Operators (user conveniences)

#Note: These commands are currently commented out since they violate the
#PostScript  model in the way the current path is consumed.
# # All complex path operators should use the primitive path and mark
# # functions. The primitive functions do assertion checking of data,
# # and maintain the graphics state. If you wing it yourself with direct
# # use of $self->strPrint( then you are throwing away the graphics state
# # support.

# sub drawLine {
#     my($self,$x0,$y0,$x1,$y1) = @_;
#     $self->getPath && message_err( 'Cannot use drawLine within a path' );
#     $self->moveto($x0,$y0);
#     $self->lineto($x1,$y1);
#     $self->stroke();
# }

# sub drawBox {
#     my($self,$sf,$x0,$y0,$x1,$y1) = @_;
  
#     $self->getPath && message_err( 'Cannot use drawLine within a path' );
#     $self->rectangle($x0,$y0,$x1-$x0,$y1-$y0);
#     my($f,$s) = ( $sf =~ /f/, $sf =~ /s/ );
#     if( $f && $s ) {
# 	$self->fillStroke;
#     } elsif( $f ) {
# 	$self->fill;
#     } elsif( $s ) {
# 	$self->stroke;
#     } else {
# 	message_err( "Parameter must contain 's' or 'f'" );
#     }
# }

# }}}
# {{{ Primitive Text Positioning Operators

# Output commands directly to pdf file. Text state is updated.

# Note that setting the text matrix will change the origin and
# scaling, as well as resetting the current point to the origin.

sub textMatrix {
    my $self = shift;
    ( assertFloat(@_) == 6 ) || message_err( "textMatrix requires 6 numbers" );
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->gupdate( 'nogstate', 'nocstate'  );
    $self->textPrint( join( ' ', af4(@_), "Tm\n" ) );
    $self->{TextX} = $self->{TextY} = 0;
}

sub textRMoveto {
    my $self = shift;
    ( assertFloat(@_) == 2 ) || message_err( "Requires 2 numbers" );
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->gupdate( 'nogstate', 'nocstate'  );
    $self->textPrint( join( ' ', af3(@_), "Td\n") );
    $self->{TextX} += $_[0];
    $self->{TextY} += $_[1];
}

sub textRMovetoLeading {
    my $self = shift;
    ( assertFloat(@_) == 2 ) || message_err( "Requires 2 numbers" );
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->gupdate( 'nogstate', 'nocstate'  );
    $self->{GNext}->setGstate( 'TL', -($_[1]) );
    $self->{GCurrent}->setGstate( 'TL', -($_[1]) );
    $self->textPrint( join( ' ', af3(@_), "TD\n") );
    $self->{TextXDiff} = $_[0];
    $self->{TextX} += $_[0];
    $self->{TextY} += $_[1];
}

sub textNextLine {
    my $self = shift;
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->gupdate( 'nogstate', 'nocstate' );
    $self->textPrint( "T*\n" );
    $self->{TextX} += $self->{TextXDiff};
    $self->{TextY} += $self->{GNext}->getTstate( 'TL' );
}


# Helper routing. Provides missing functionality of moveto in text
# space. Depends on functions that update $textX and $textY.

sub textMoveto {
    my $self = shift;
    my($x,$y) = @_;
    ( assertFloat($x,$y) == 2 ) || message_err( "One of $x or $y is not a float" );
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->gupdate( 'nogstate', 'nocstate' );
    $self->textPrint( join( ' ', af3($x-$self->{TextX},$y-$self->{TextY}), "Td\n") );
    $self->{TextX} = $x;
    $self->{TextY} = $y;
}

# }}}
# {{{ Primitive Text State Operators

# Local helper methods

sub forceTstate {
    my $self = shift;
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->{GNext}->setTstate( @_ );
    $self->{GCurrent}->setTstate( @_ );
    $self->{GNext}->printTstate( $_[0] );
}

# Text state operators

sub setCharSpace {
    my $self = shift;
    assertFloat(@_) == 1 || message_err( 'Not a number' );
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->{GNext}->setTstate( 'Tc', @_ );
}

sub setTextLeading {
    my $self = shift;
    assertFloat(@_) == 1 || message_err( 'Not a number' );
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->{GNext}->setTstate( 'TL', @_ );
}

sub setTextRender {
    my $self = shift;
    assertFloat(@_) == 1 || message_err( 'Not a number' );
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->{GNext}->setTstate( 'Tr', @_ );
}

sub setTextRise {
    my $self = shift;
    assertFloat(@_) == 1 || message_err( 'Not a number' );
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->{GNext}->setTstate( 'Ts', @_ );
}

sub setTextWordSpace {
    my $self = shift;
    assertFloat(@_) == 1 || message_err( 'Not a number' );
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->{GNext}->setTstate( 'Tw', @_ );
}

sub setTextHorizScale {
    my $self = shift;
    assertFloat(@_) == 1 || message_err( 'Not a number' );
    $self->getPath && message_err( 'Cannot do operation within a path' );
    $self->{GNext}->setTstate( 'Tz', @_ );
}

sub forceCharSpace {
    my $self = shift;
    assertFloat(@_) == 1 || message_err( 'Not a number' );
    $self->forceTstate( 'Tc', @_ );
}

sub forceTextLeading {
    my $self = shift;
    assertFloat(@_) == 1 || message_err( 'Not a number' );
    $self->forceTstate( 'TL', @_ );
}

sub forceTextRender {
    my $self = shift;
    assertFloat(@_) == 1 || message_err( 'Not a number' );
    $self->forceTstate( 'Tr', @_ );
}

sub forceTextRise {
    my $self = shift;
    assertFloat(@_) == 1 || message_err( 'Not a number' );
    $self->forceTstate( 'Ts', @_ );
}

sub forceTextWordSpace {
    my $self = shift;
    assertFloat(@_) == 1 || message_err( 'Not a number' );
    $self->forceTstate( 'Tw', @_ );
}

sub forceTextHorizScale {
    my $self = shift;
    assertFloat(@_) == 1 || message_err( 'Not a number' );
    $self->forceTstate( 'Tz', @_ );
}

# }}}
# {{{ Primitive Font Text State Operators

# Change the current text font and/or font size.
#   setFont( 'Helvetica-Bold' ) sets font to Helvetica-Bold
#   setFont( 'Courier', 12 ) sets font to Courier and fontsize to 12 pt
#   setFont( 8 ) sets current font size to 8 pt (fontname unaffected).

sub setFont {
    my $self = shift;
    my(@p) = @_;

    $self->getPath && message_err( 'Cannot do operation within a path' );
    if( assertFloat($p[0]) ) {
	$p[0] > 0 || message_err( 'Font size must be greater then 0' );
	$self->{GNext}->setFontSize( $p[0] );
	if( defined($p[1]) ) {
	    $self->{File}->fontRegister( $p[1] );
	    $self->{GNext}->setFontName( $p[1] );
	    $self->useFont( $p[1] );
	}
    } else {
	$self->{File}->fontRegister( $p[0] );
	$self->{GNext}->setFontName( $p[0] );
	$self->useFont( $p[0] );
	if( defined($p[1]) ) {
	    assertFloat($p[1]) || message_err( "Expecting number, found '$p[1]'" );
	    $p[1] > 0 || message_err( 'Font size must be greater then 0' );
	    $self->{GNext}->setFontSize( $p[1] );
	}
    }
}

sub forceFont {
    my $self = shift;
    my(@p) = @_;

    $self->setFont( @_ );
    $self->{GCurrent}->setFontName( $self->{GNext}->getFontName );
    $self->{GCurrent}->setFontSize( $self->{GNext}->getFontSize );
    $self->{GNext}->printFont( );
}

# }}}
# {{{ Primitive Text Show Operators


sub show {
    my $self = shift;
    my($text) = @_;
    $self->gupdate;
    $self->textPrint( '(', pdfEscape($text), ")Tj\n" );
}

sub nextlineShow {
    my $self = shift;
    my($text) = @_;
    $self->gupdate;
    $self->textPrint( '(', pdfEscape($text), ")\'\n" );
    $self->{TextX} += $self->{GNext}->getTstate( 'TL' );
}

sub xyNextlineShow {
    my $self = shift;
    my($aw,$ac,$text) = @_;
    assertFloat($ac,$aw) == 2  || message_err( "xyNextlineShow requires 2 numbers" );
    $self->gupdate;
    $self->setTextWordSpace( $aw );
    $self->setCharSpace( $ac );
    $self->textPrint( '(', pdfEscape($text), ")\"\n" );
    $self->{TextX} += $self->{GNext}->getTstate( 'TL' );
}

sub showKerning {
    my $self = shift;

    $self->gupdate;
    $self->textPrint( '[' );
    my $string = 1;
    my $p;
    foreach $p (@_) {
	if( $string ) {
	    $self->textPrint( '(', pdfEscape($p), ') ' );
	    $string &= 0;
	} else {
	    assertFloat($p) == 1 || message_err( "Expecting number, found '$p'" );
	    $self->textPrint( f3($p), ' ' );
	    $string |= 1;
	}
    }
    $self->textPrint( "]TJ\n" );
}

# }}}
# {{{ Complex Text Rendering using Text Matrix

# Text drawing routines using Text Matrix for setting position and
# font size. Require x,y coordinate, font size and $text string. The
# font type is set with setFont(). Text strings should only use
# backslash escape to add octal character codes. Do not backslash
# backslashes or left/right parenthesis: these will be backslashed
# automatically.

sub tmTextLeft {
    my $self = shift;
    my($x,$y,$fsize,$text) = @_;
    
    $self->setFont( $fsize );
    $self->textMatrix( 1, 0, 0, 1, $x, $y );
    $self->show( $text );
}


sub tmTextRight {
    my $self = shift;
    my($x,$y,$fsize,$text) = @_;
    $self->setFont( $fsize );
    $x = $x - $self->textWidth($text);	# Uses width in next gstate
    $self->tmTextLeft($x,$y,$fsize,$text);
}


sub tmTextCenter {
    my $self = shift;
    my($x,$y,$fsize,$text) = @_;
    $self->setFont( $fsize );
    $x = $x - 0.5 * $self->textWidth($text);
    $self->tmTextLeft($x,$y,$fsize,$text,1);
}

# }}}
# {{{ Complex Text Rendering without Text Matrix

sub textLeft {
    my $self = shift;
    my($x,$y,$text) = @_;
    
    $self->textMoveto( $x, $y );
    $self->show( $text );
}


sub textRight {
    my $self = shift;
    my($x,$y,$text) = @_;
    $x = $x - $self->textWidth($text);
    $self->textLeft($x,$y,$text);
}


sub textCenter {
    my $self = shift;
    my($x,$y,$text) = @_;
    $x = $x - 0.5 * $self->textWidth($text);
    $self->textLeft($x,$y,$text,1);
}

# Creates a link around the text. Destinations must be predefined with addDest().




# {{{ nuking annots related functions 

# sub textCenterLink {
#     my $self = shift;
#     my($x,$y,$dest,$color,$text) = @_;
#     my $w = $self->textWidth($text);
#     my $ydiff = $self->{GNext}->getFontSize( );
#     $self->{File}->addLink( $x-0.5*$w, $y - 0.2*$ydiff, $x+0.5*$w, $y + 0.8 * $ydiff,
# 			   $dest, $color );
#     $self->textLeft($x-0.5*$w,$y,$text);
# }

# sub textLeftLink {
#     my $self = shift;
#     my($x,$y,$dest,$color,$text) = @_;
#     my $w = $self->textWidth($text);
#     my $ydiff = $self->{GNext}->getFontSize( );
#     $self->{File}->addLink( $x, $y - 0.2*$ydiff, $x+$w, $y + 0.8 * $ydiff,
# 			   $dest, $color );
#     $self->textLeft($x,$y,$text);
# }


# sub textRightLink {
#     my $self = shift;
#     my($x,$y,$dest,$color,$text) = @_;
#     my $w = $self->textWidth($text);
#     my $ydiff = $self->{GNext}->getFontSize( );
#     $self->{File}->addLink( $x-$w, $y - 0.2*$ydiff, $x, $y + 0.8 * $ydiff,
# 			   $dest, $color );
#     $self->textLeft($x-$w,$y,$text);
# }

# }}}

# }}}
# {{{ Complex Text Rendering using blocks

# These routines may be moved out to a separate package

# Self contained text drawing routines (you do not need to begin or
# end the text stream: just make one call to write the text).

sub textBlockLeft {
    my $self = shift;
    my($x,$y,$fontname,$fontsize,$leading,@textlist) = @_;

    $self->beginText( $fontname, $fontsize );

    $self->textRMoveto( $x, $y + $leading );
    $self->setTextLeading( $leading );

    my $text;
    foreach $text (@textlist) {
	$self->nextlineShow( $text );
    }
    $self->endText();
}

sub textBlockRight { 
    my $self = shift;
    $self->textBlockOffset(1.0,@_);
}

sub textBlockCenter {
    my $self = shift;
    $self->textBlockOffset(0.5,@_);
}

sub textBlockOffset {
    my $self = shift;
    my($xoffset,$x,$y,$fontname,$fontsize,$leading,@textlist) = @_;
    
    $self->beginText( $fontname, $fontsize );

    my($text);
    my($width0,$width1) = ( $x, 0 );
    foreach $text (@textlist) {
	$width1 = $xoffset * $self->textWidth($text);
	$x = ( $width0 - $width1 );
	$width0 = $width1;
	$self->textRMoveto( $x, $y );
	$y = -$leading;
	$self->show( $text );
    }
    $self->endText();
}

# }}}
# {{{ String Width routines

# Calculates width of string $str using current graphics state to
# determine text spacing and scaling.

sub textWidth {
    my $self = shift;

    my $fontref = $self->{File}->getFontRef( $self->{GNext}->getFontName() );
    $fontref->textWidth( $self->{GNext}->getFontSize,
			$self->{GNext}->getTstate( 'Tc' ),
			$self->{GNext}->getTstate( 'Tw' ),
			$self->{GNext}->getTstate( 'Tz' ),
			@_ );
}

# }}}
# {{{ Color

# Sets the color and/or colorspace in the $gstate_next object. This
# color will only get written to the pdf file when it needs to be,
# i.e. on the next draw operator. Example calls are:
#	&setColor( 'stroke', 'red' );
#	&setColor( 'fill', 'DeviceRGB', 1, 0, 0 );
#	&setColor( 'fill', 1, 0, 0 );
# In the examples above, the first example uses a predefined
# color. The second example specifies a colorspace and color
# value. The third example assumes no change in color space and just
# specified a color value. &setColor as shown in the third example
# is the method required to change colorspace. Colorspace cannot be
# changed without also providing an array of values.

sub setColor {
    my $self = shift;		# file
    my($sf,@p) = @_;

    $self->getPath && message_err( 'Cannot do operation within a path' );
    my $cmapref = $self->{File}->{ColorMap};

    # Check that first parameter is 'stroke' or 'fill'

    ( $sf eq 'stroke' || $sf eq 'fill' ) || 
	message_err( "Requires 'stroke' or 'fill' as first parameter" );
    (@p) || message_err( "Insufficient parameters" );

    # Check if first @p parameter is a defined color. If so then set
    # the color name in the next gstate object for each page.

    my $newColorSpace = $p[0];
    if( defined($cmapref->{$p[0]}{ColorSpace}) ) {
	$self->{GNext}->setColorName( $sf, $p[0] );
	$self->{GNext}->setColorSpace( $sf, $cmapref->{$p[0]}{ColorSpace} );
	$self->{GNext}->setColorValue( $sf, $cmapref->{$p[0]}{Value} );
	$self->{GNext}->change( 'cstate' );

    # This is not a predefined color, so more work is necessary.

    } else {

	# Check if first remaining parameter is a defined
	# colorspace. If it is and there are no remaining parameters
	# then tell user that a seed value is required for color value.

	my $csref = $self->{File}->{ColorSpace}{$p[0]};
	if( defined( $csref ) ) {
	    shift @p;
	    if( scalar(@p) < 1 ) {
		message_err( "Must specify a color when changing colorspace" );
	    }
	} else {

	    # We need to know the colorspace. Use the value in next gstate

	    $csref = $self->{File}->getColorSpaceRef( $self->{GNext}->getColorSpace( $sf ) );
	}
    
	eval{ $csref->refType } eq 'PdfColorSpace' ||
	    message_err( 'Internal -- No object reference found for ColorSpace' );
	
	# Check if remaining parameters define a valid color value array.
	assertFloatRange( 0, 1, @p ) == $csref->getFlag( 'Dimen' ) || 
	    message_err( "Invalid numbers, or incorrect array depth for colorspace $newColorSpace " );

	# Set the new colorspace and color value in $gstate_next,
	# even though these may be the same as the existing
	# values. gstate->update will do the real checking to see
	# if any color changes are made.

	$self->{GNext}->setColorName( $sf, '' );
	$self->{GNext}->setColorSpace( $sf, $csref->getFlag( 'ColorSpace' ) );
	$self->{GNext}->setColorValue( $sf, \@p );
	# jpravetz - 9apr96 - added this line
	$self->{GNext}->change( 'cstate' );
    }

}

# Force the color to be printed to the file, regardless of gstate.

sub forceColor {
    my $self = shift;
    my $sf = shift;

    $self->setColor( $sf, @_ );
    $self->{GCurrent}->setColorName( $sf, $self->{GNext}->getColorName( $sf ) );
    $self->{GCurrent}->setColorSpace( $sf, $self->{GNext}->getColorSpace( $sf ) );
    $self->{GCurrent}->setColorValue( $sf, $self->{GNext}->getColorValue( $sf ) );
    $self->printColor( $sf );
}

# Shortcut for method setColor. Parameters are as per method setColor
# except $sf can contain either or both of the characters 's' and 'f'
# for stroke and fill.

sub setColors {
    my $self = shift;		# file
    my($sf,@p) = @_;
    my $status = 0;

    if( $sf =~ /s/i ) {
	$self->setColor( 'stroke', @p );
	++$status;
    }
    if( $sf =~ /f/i ) {
	$self->setColor( 'fill', @p );
	++$status;
    }
    $status || message_err( "First parameter contains neither 's' or 'f'" );
}

sub forceColors {
    my $self = shift;		# file
    my($sf,@p) = @_;
    my $status = 0;

    if( $sf =~ /s/i ) {
	$self->forceColor( 'stroke', @p );
	++$status;
    }
    if( $sf =~ /f/i ) {
	$self->forceColor( 'fill', @p );
	++$status;
    }
    $status || message_err( "First parameter contains neither 's' or 'f'" );
}

# }}}
# {{{ Emacs local variables  

### Local variables:
### major-mode: perl-mode
### perl-indent-level: 4
### voice-lock-mode: t
### folded-file: t
### End:

# }}}
1;
